#ifndef _VisibilityPreprocessingManager_H__
#define _VisibilityPreprocessingManager_H__

#include "GtpVisibilitySceneTraverser.h"
#include "GtpVisibilityInfo.h"


/** This abstract class defining interface for a specific visibility query
    algorithm. The algorithm is either used to render the scene with false colors
    and query visibility of scene nodes and polygons */
*/
class VisibilityPreprocessingInterface
{
public:


  /** Constructor taking a scene traverser for a specific type of hierarchy as argument. 
   */
	VisibilityPreprocessingInterface( GtpVisibility::SceneTraverser *sceneTraverser );
  
  
  /** 
      Export the scene for visibility preprocessing. Exports the hierarchy including
      its bounding boxes + mesh data for occluders. The viewcell data will either be
      supplied directly to the visibility preprocessing standalone module
  */
  virtual ErrorCode ExportScene(const string filename);
  
  
  /** 
      Load preprocessed visibility information
  */
  virtual ErrorCode LoadPreprocessedData(const string filename);
  
  /** 
      Uses the specified sphere to find its PVS
  */
  virtual ErrorCode GetPointPVS(const Vector3D &point,
				VisibilityContainer<NodeVisibilityInfo> *visibleNodes,
				VisibilityContainer<MeshVisibilityInfo> *visibleMeshes ) = 0;
  
  /** 
      Uses the specified sphere to find its PVS
  */
  virtual ErrorCode GetSpherePVS(const Vector3D &point,
				 const float radius,
				 VisibilityContainer<NodeVisibilityInfo> *visibleNodes,
				 VisibilityContainer<MeshVisibilityInfo> *visibleMeshes ) = 0;

  
  
  /** Sets the scene traverser.
      @remark the scene traverser is dependent on the type of hierarchy the scene consists of.
  */
  void setSceneTraverser( GtpVisibility::SceneTraverser *sceneTraverser );

protected:
  /** Helper method for getting a scene name */
  string GetSceneName() const;

  /** Get checksum of the current occluder set in the scene graph. This method is used to check
      if the preprocessed data matches the current scene graph
      -> $$ could be moved to VisibilitySceneTraverser */
  long GetOccluderChecksum() const;

  /** Get checksum of the current occludee set in the scene graph. This method is used to check
      if the preprocessed data matches the current scene graph
      -> $$ could be moved to VisibilitySceneTraverser */
  long GetOccludeeChecksum() const;

  /** 
      Finds a viewcell given a point in space
  */
  virtual ErrorCode FindViewCellId(const Vector3D &point,
				   int *viewCellId
				   );

  /** 
      Uses the specified viewcell to find its PVS
  */
  virtual ErrorCode GetViewCellPVS(const int cellID,
				   VisibilityContainer<NodeVisibilityInfo> *visibleNodes,
				   VisibilityContainer<MeshVisibilityInfo> *visibleMeshes ) = 0;

  GtpVisibility::SceneTraverser *mSceneTraverser;
};

#endif // VisibilityPreprocessingManager
