#ifndef _VisibilityQueryManager_H__
#define _VisibilityQueryManager_H__

#include "VisibilityInfo.h"


/** This abstract class defining interface for a specific visibility query
    algorithm. The algorithm is either used to render the scene with false colors
    and query visibility of scene nodes and polygons */
*/
class VisibilityQueryManager
{
public:
  /** Constructor taking a scene traverser for a specific type of hierarchy as argument. 
   */
	VisibilityQueryManager( GtpVisibility::SceneTraverser *sceneTraverser );

  /** 
      Uses explicit camera to execute the visibility query
  */
  virtual void
  ComputeFromPointVisibility(const Camera &camera,
			     VisibilityContainer<NodeVisibilityInfo> *visibleNodes,
			     VisibilityContainer<MeshVisibilityInfo> *visibleGeometry ) = 0;
  
  /** 
      Uses the specified point to execute the visibility query in all directions
  */
  virtual void
  ComputeFromPointVisibility(const Vector3D &point,
			     VisibilityContainer<NodeVisibilityInfo> *visibleNodes,
			     VisibilityContainer<MeshVisibilityInfo> *visibleGeometry ) = 0;
  
  /** 
      Ray shooting intreface: finds an intersection with the first opaque
      object in the scene if globalLine = false, otherwise finds all intersections
      with the scene. 
  */
  virtual void
  ShootRay(const Ray &ray,
	   VisibilityContainer<NodeVisibilityInfo> *visibleNodes,
	   VisibilityContainer<MeshVisibilityInfo> *visibleMeshes,
	   bool isGlobalLine = false
	   ) = 0;
  
  
  /** 
      Preprocessing interface: PVS for a given point found using PVS of the corresponding viewcell
  */
  virtual int
  GetFromPointPVS(const Vector3D &point,
		  VisibilityContainer<NodeVisibilityInfo> *visibleNodes,
		  VisibilityContainer<MeshVisibilityInfo> *visibleMeshes ) = 0;
  
  /** 
      Preprocessing interface: PVS for a given sphere using union of cell based PVS
  */
  virtual int
  GetFromSpherePVS(const Vector3D &center,
		   const float radius,
		   VisibilityContainer<NodeVisibilityInfo> *visibleNodes,
		   VisibilityContainer<MeshVisibilityInfo> *visibleMeshes ) = 0;
  
  
  /** Sets the scene traverser.
      @remark the scene traverser is dependent on the type of hierarchy the scene consists of.
  */
  void setSceneTraverser( GtpVisibility::SceneTraverser *sceneTraverser );

protected:
	GtpVisibility::SceneTraverser *mSceneTraverser;
  
  VisibilityPreprocessingInterface *mPreprocessingManager;
};

#endif // VisibilityQueryManager
